//
// (C) Copyright 2005 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Collections;
using System.Diagnostics;

using Autodesk.Revit;
using Autodesk.Revit.Collections;

using RvtMgdDbg.Snoop.Collectors;

namespace RvtMgdDbg.Snoop.CollectorExts
{
	/// <summary>
	/// Provide Snoop.Data for any classes related to an Application.
	/// </summary>
	
	public class CollectorExtApp : CollectorExt
	{
		public
		CollectorExtApp()
		{
		}

        protected override void
        CollectEvent(object sender, CollectorEventArgs e)
        {
                // cast the sender object to the SnoopCollector we are expecting
            Collector snoopCollector = sender as Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }

                // see if it is a type we are responsible for
			Autodesk.Revit.Application app = e.ObjToSnoop as Autodesk.Revit.Application;
			if (app != null) {
				Stream(snoopCollector.Data(), app);
				return;
			}

			Autodesk.Revit.Options.Application appOptions = e.ObjToSnoop as Autodesk.Revit.Options.Application;
			if (appOptions != null) {
				Stream(snoopCollector.Data(), appOptions);
				return;
			}

            Autodesk.Revit.ExternalCommandData extCmd = e.ObjToSnoop as Autodesk.Revit.ExternalCommandData;
            if (extCmd != null) {
                Stream(snoopCollector.Data(), extCmd);
                return;
            }

            MenuItem menuItem = e.ObjToSnoop as MenuItem;
            if (menuItem != null) {
                Stream(snoopCollector.Data(), menuItem);
                return;
            }

            Toolbar toolbar = e.ObjToSnoop as Toolbar;
            if (toolbar != null) {
                Stream(snoopCollector.Data(), toolbar);
                return;
            }

            ToolbarItem toolbarItem = e.ObjToSnoop as ToolbarItem;
            if (toolbarItem != null) {
                Stream(snoopCollector.Data(), toolbarItem);
                return;
            }

            ParameterStorage paramStorage = e.ObjToSnoop as ParameterStorage;
            if (paramStorage != null) {
                Stream(snoopCollector.Data(), paramStorage);
                return;
            }
        }
        
		private void
		Stream(ArrayList data, Autodesk.Revit.Application app)
		{
		    data.Add(new Snoop.Data.ClassSeparator(typeof(Autodesk.Revit.Application)));

            data.Add(new Snoop.Data.Object("Active document", app.ActiveDocument));           
            data.Add(new Snoop.Data.Object("Cities", app.Cities));
            data.Add(new Snoop.Data.Enumerable("Documents", app.Documents));
            data.Add(new Snoop.Data.Bool("Is quiescent", app.IsQuiescent));
            data.Add(new Snoop.Data.Enumerable("Loaded applications", app.LoadedApplications));
            data.Add(new Snoop.Data.Object("Object factory", app.ObjectFactory));
            data.Add(new Snoop.Data.Object("Options", app.Options));
            data.Add(new Snoop.Data.String("Recording journal filename", app.RecordingJournalFilename));
            data.Add(new Snoop.Data.String("Version build", app.VersionBuild));
            data.Add(new Snoop.Data.String("Version name", app.VersionName));
            data.Add(new Snoop.Data.String("Version number", app.VersionNumber));

            Autodesk.Revit.Collections.Array menus = new Autodesk.Revit.Collections.Array();
            app.GetAllTopMenus(ref menus);
            data.Add(new Snoop.Data.Enumerable("Top menus", menus));

            data.Add(new Snoop.Data.Enumerable("Toolbars", app.GetToolbars()));            
        }

		private void
		Stream(ArrayList data, Autodesk.Revit.Options.Application appOptions)
		{
		    data.Add(new Snoop.Data.ClassSeparator(typeof(Autodesk.Revit.Options.Application)));

            data.Add(new Snoop.Data.String("Shared parameters filename", appOptions.SharedParametersFilename));

		    data.Add(new Snoop.Data.CategorySeparator("Library Paths"));

			StringStringMapIterator iter = appOptions.LibraryPaths.ForwardIterator();
			while (iter.MoveNext())
				data.Add(new Snoop.Data.String(iter.Key, iter.Current.ToString()));
        }

        private void
        Stream(ArrayList data, Autodesk.Revit.ExternalCommandData extCmd)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(Autodesk.Revit.ExternalCommandData)));

            data.Add(new Snoop.Data.Object("Application", extCmd.Application));
            data.Add(new Snoop.Data.Enumerable("Data", extCmd.Data));
            data.Add(new Snoop.Data.Object("View", extCmd.View));
        }

        private void
        Stream(ArrayList data, MenuItem menuItem)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(MenuItem)));

            data.Add(new Snoop.Data.String("Name", menuItem.Name));
            data.Add(new Snoop.Data.String("Assembly name", menuItem.AssemblyName));
            data.Add(new Snoop.Data.String("Class name", menuItem.ClassName));
            data.Add(new Snoop.Data.Bool("Checked", menuItem.Checked));
            data.Add(new Snoop.Data.Bool("Enabled", menuItem.Enabled));
            data.Add(new Snoop.Data.String("Menu item type", menuItem.MenuItemType.ToString()));
            data.Add(new Snoop.Data.String("Status bar tip", menuItem.StatusbarTip));
        }

        private void
        Stream(ArrayList data, Toolbar toolbar)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(Toolbar)));

            data.Add(new Snoop.Data.String("Name", toolbar.Name));
            data.Add(new Snoop.Data.String("Image", toolbar.Image));
            data.Add(new Snoop.Data.Bool("Visible", toolbar.Visible));
            data.Add(new Snoop.Data.Bool("Enabled", toolbar.Enabled));
            data.Add(new Snoop.Data.Enumerable("Toolbar items", toolbar.ToolbarItems));
        }

        private void
        Stream(ArrayList data, ToolbarItem tbItem)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(ToolbarItem)));

            data.Add(new Snoop.Data.String("Assembly name", tbItem.AssemblyName));
            data.Add(new Snoop.Data.String("Class name", tbItem.ClassName));
            data.Add(new Snoop.Data.Bool("Checked", tbItem.Checked));
            data.Add(new Snoop.Data.Bool("Enabled", tbItem.Enabled));
            data.Add(new Snoop.Data.String("Image index", tbItem.ImageIndex.ToString()));
            data.Add(new Snoop.Data.String("Item type", tbItem.ItemType.ToString()));
            data.Add(new Snoop.Data.String("Item text", tbItem.ItemText));
            data.Add(new Snoop.Data.String("Status bar tip", tbItem.StatusbarTip));
            data.Add(new Snoop.Data.String("Tool tip", tbItem.ToolTip));
        }

        private void
        Stream(ArrayList data, ParameterStorage paramStorage)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(ParameterStorage)));

            data.Add(new Snoop.Data.String("Storage type", paramStorage.StorageType.ToString()));
        }
	}
}
