//
// (C) Copyright 2005 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Collections;
using System.Diagnostics;

using Autodesk.Revit;
using Autodesk.Revit.Elements;
using Autodesk.Revit.Events;

using RvtMgdDbg.Snoop.Collectors;

namespace RvtMgdDbg.Snoop.CollectorExts {
    /// <summary>
    /// Provide Snoop.Data for any classes related to an Application.
    /// </summary>

    public class CollectorExtMisc : CollectorExt
    {
        public
        CollectorExtMisc()
        {
        }

        protected override void
        CollectEvent(object sender, CollectorEventArgs e)
        {
                // cast the sender object to the SnoopCollector we are expecting
            Collector snoopCollector = sender as Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }

                // see if it is a type we are responsible for
            Autodesk.Revit.Color color = e.ObjToSnoop as Autodesk.Revit.Color;
            if (color != null) {
                Stream(snoopCollector.Data(), color);
                return;
            }

            LayoutRule layoutRule = e.ObjToSnoop as LayoutRule;
            if (layoutRule != null) {
                Stream(snoopCollector.Data(), layoutRule);
                return;
            }

            FormatOptions formatOptions = e.ObjToSnoop as FormatOptions;
            if (formatOptions != null) {
                Stream(snoopCollector.Data(), formatOptions);
                return;
            }

            CurtainGrid curtainGrid = e.ObjToSnoop as CurtainGrid;
            if (curtainGrid != null) {
                Stream(snoopCollector.Data(), curtainGrid);
                return;
            }

            CurtainCell curtainCell = e.ObjToSnoop as CurtainCell;
            if (curtainCell != null) {
                Stream(snoopCollector.Data(), curtainCell);
                return;
            }

            RebarHostData rebarHostData = e.ObjToSnoop as RebarHostData;
            if (rebarHostData != null) {
                Stream(snoopCollector.Data(), rebarHostData);
                return;
            }

            Leader leader = e.ObjToSnoop as Leader;
            if (leader != null) {
                Stream(snoopCollector.Data(), leader);
                return;
            }

            Filter filter = e.ObjToSnoop as Filter;
            if (filter != null) {
                Stream(snoopCollector.Data(), filter);
                return;
            }

            PaperSize paperSize = e.ObjToSnoop as PaperSize;
            if (paperSize != null) {
                Stream(snoopCollector.Data(), paperSize);
                return;
            }

            PaperSource paperSource = e.ObjToSnoop as PaperSource;
            if (paperSource != null) {
                Stream(snoopCollector.Data(), paperSource);
                return;
            }

            PrintSetup prnSetup = e.ObjToSnoop as PrintSetup;
            if (prnSetup != null) {
                Stream(snoopCollector.Data(), prnSetup);
                return;
            }

            PrintParameters prnParams = e.ObjToSnoop as PrintParameters;
            if (prnParams != null) {
                Stream(snoopCollector.Data(), prnParams);
                return;
            }

            PostedError postErr = e.ObjToSnoop as PostedError;
            if (postErr != null) {
                Stream(snoopCollector.Data(), postErr);
                return;
            }

            VolumeCalculationSetting volCalc = e.ObjToSnoop as VolumeCalculationSetting;
            if (volCalc != null) {
                Stream(snoopCollector.Data(), volCalc);
                return;
            }

            VolumeCalculationOptions volCalcOpts = e.ObjToSnoop as VolumeCalculationOptions;
            if (volCalcOpts != null) {
                Stream(snoopCollector.Data(), volCalcOpts);
                return;
            }

            ViewSheetSetting viewSheetSetting = e.ObjToSnoop as ViewSheetSetting;
            if (viewSheetSetting != null) {
                Stream(snoopCollector.Data(), viewSheetSetting);
                return;
            }

            DialogBoxData dlgBoxData = e.ObjToSnoop as DialogBoxData;
            if (dlgBoxData != null) {
                Stream(snoopCollector.Data(), dlgBoxData);
                return;
            }                      
        }

        private void
        Stream(ArrayList data, Autodesk.Revit.Color color)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(Autodesk.Revit.Color)));

            data.Add(new Snoop.Data.Int("Red", color.Red));
            data.Add(new Snoop.Data.Int("Green", color.Green));
            data.Add(new Snoop.Data.Int("Blue", color.Blue));
        }

        private void
        Stream(ArrayList data, LayoutRule layoutRule)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(LayoutRule)));

            // no data at this level

            LayoutRuleClearSpacing lrClearSpacing = layoutRule as LayoutRuleClearSpacing;
            if (lrClearSpacing != null) {
                Stream(data, lrClearSpacing);
                return;
            }

            LayoutRuleFixedDistance lrFixedDist = layoutRule as LayoutRuleFixedDistance;
            if (lrFixedDist != null) {
                Stream(data, lrFixedDist);
                return;
            }

            LayoutRuleFixedNumber lrFixedNum = layoutRule as LayoutRuleFixedNumber;
            if (lrFixedNum != null) {
                Stream(data, lrFixedNum);
                return;
            }

            LayoutRuleMaximumSpacing lrMaxSpacing = layoutRule as LayoutRuleMaximumSpacing;
            if (lrMaxSpacing != null) {
                Stream(data, lrMaxSpacing);
                return;
            }
        }

        private void
        Stream(ArrayList data, LayoutRuleClearSpacing layoutRule)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(LayoutRuleClearSpacing)));

            data.Add(new Snoop.Data.String("Justify type", layoutRule.JustifyType.ToString()));
            data.Add(new Snoop.Data.Double("Spacing", layoutRule.Spacing));
        }

        private void
        Stream(ArrayList data, LayoutRuleFixedDistance layoutRule)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(LayoutRuleFixedDistance)));

            data.Add(new Snoop.Data.String("Justify type", layoutRule.JustifyType.ToString()));
            data.Add(new Snoop.Data.Double("Spacing", layoutRule.Spacing));
        }

        private void
        Stream(ArrayList data, LayoutRuleFixedNumber layoutRule)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(LayoutRuleFixedNumber)));

            data.Add(new Snoop.Data.Int("Number of lines", layoutRule.NumberOfLines));
        }

        private void
        Stream(ArrayList data, LayoutRuleMaximumSpacing layoutRule)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(LayoutRuleMaximumSpacing)));

            data.Add(new Snoop.Data.Double("Spacing", layoutRule.Spacing));
        }

        private void
        Stream(ArrayList data, FormatOptions opt)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(FormatOptions)));

            data.Add(new Snoop.Data.String("Name", opt.Name));
            data.Add(new Snoop.Data.String("Unit symbol", opt.Unitsymbol.ToString()));
            data.Add(new Snoop.Data.String("Units", opt.Units.ToString()));
            data.Add(new Snoop.Data.Object("Group", opt.Group));     // TBD: strangely named property
            data.Add(new Snoop.Data.Double("Rounding", opt.Rounding));
        }

        private void
        Stream(ArrayList data, CurtainGrid grid)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(CurtainGrid)));

            data.Add(new Snoop.Data.Enumerable("Cells", grid.Cells));
            data.Add(new Snoop.Data.Enumerable("Mullions", grid.Mullions));
            data.Add(new Snoop.Data.Enumerable("Unlocked mullions", grid.UnlockedMullions));
            data.Add(new Snoop.Data.Angle("Grid 1 angle", grid.Grid1Angle));
            data.Add(new Snoop.Data.String("Grid 1 justification", grid.Grid1Justification.ToString()));
            data.Add(new Snoop.Data.Double("Grid 1 offset", grid.Grid1Offset));
            data.Add(new Snoop.Data.Angle("Grid 2 angle", grid.Grid2Angle));
            data.Add(new Snoop.Data.String("Grid 2 justification", grid.Grid2Justification.ToString()));
            data.Add(new Snoop.Data.Double("Grid 2 offset", grid.Grid2Offset));           
            data.Add(new Snoop.Data.Int("Number of panels", grid.NumPanels));
            data.Add(new Snoop.Data.Enumerable("Panels", grid.Panels));
            data.Add(new Snoop.Data.Enumerable("Unlocked panels", grid.UnlockedPanels));
            data.Add(new Snoop.Data.Int("Number of U lines", grid.NumULines));
            data.Add(new Snoop.Data.Enumerable("U grid lines", grid.UGridLines));
            data.Add(new Snoop.Data.Int("Number of V lines", grid.NumVLines));
            data.Add(new Snoop.Data.Enumerable("V grid lines", grid.VGridLines));
        }

        private void
        Stream(ArrayList data, CurtainCell cell)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(CurtainCell)));

            data.Add(new Snoop.Data.Enumerable("Curve loops", cell.CurveLoops));
            data.Add(new Snoop.Data.Enumerable("Planarized curve loops", cell.PlanarizedCurveLoops));
        }

        private void
        Stream(ArrayList data, RebarHostData rebarHostData)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(RebarHostData)));

            data.Add(new Snoop.Data.Bool("Valid", rebarHostData.Valid));            
        }

        private void
        Stream(ArrayList data, Leader leader)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(Leader)));

            data.Add(new Snoop.Data.Xyz("Elbow", leader.Elbow));
            data.Add(new Snoop.Data.Xyz("End", leader.End));
        }

        private void
        Stream(ArrayList data, Filter filter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(Filter)));

            // Nothing at this level yet!

            FamilyFilter famFilter = filter as FamilyFilter;
            if (famFilter != null) {
                Stream(data, famFilter);
                return;
            }

            LogicAndFilter logicAndFilter = filter as LogicAndFilter;
            if (logicAndFilter != null) {
                Stream(data, logicAndFilter);
                return;
            }

            LogicNotFilter logicNotFilter = filter as LogicNotFilter;
            if (logicNotFilter != null) {
                Stream(data, logicNotFilter);
                return;
            }

            LogicOrFilter logicOrFilter = filter as LogicOrFilter;
            if (logicOrFilter != null) {
                Stream(data, logicOrFilter);
                return;
            }

            MaterialFilter matFilter = filter as MaterialFilter;
            if (matFilter != null) {
                Stream(data, matFilter);
                return;
            }

            ParameterFilter paramFilter = filter as ParameterFilter;
            if (paramFilter != null) {
                Stream(data, paramFilter);
                return;
            }

            StructuralTypeFilter structFilter = filter as StructuralTypeFilter;
            if (structFilter != null) {
                Stream(data, structFilter);
                return;
            }

            SymbolFilter symFilter = filter as SymbolFilter;
            if (symFilter != null) {
                Stream(data, symFilter);
                return;
            }

            TypeFilter typeFilter = filter as TypeFilter;
            if (typeFilter != null) {
                Stream(data, typeFilter);
                return;
            }

            WallUsageFilter wallFilter = filter as WallUsageFilter;
            if (wallFilter != null) {
                Stream(data, wallFilter);
                return;
            }
        }

        private void
        Stream(ArrayList data, FamilyFilter famFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(FamilyFilter)));

            // Nothing at this level yet!           
        }

        private void
        Stream(ArrayList data, LogicAndFilter logicAndFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(LogicAndFilter)));

            data.Add(new Snoop.Data.Object("Left filter", logicAndFilter.LeftFilter));
            data.Add(new Snoop.Data.Object("Right filter", logicAndFilter.RightFilter));
        }

        private void
        Stream(ArrayList data, LogicNotFilter logicNotFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(LogicNotFilter)));

            data.Add(new Snoop.Data.Object("Sub filter", logicNotFilter.SubFilter));            
        }

        private void
        Stream(ArrayList data, LogicOrFilter logicOrFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(LogicOrFilter)));

            data.Add(new Snoop.Data.Object("Left filter", logicOrFilter.LeftFilter));
            data.Add(new Snoop.Data.Object("Right filter", logicOrFilter.RightFilter));
        }

        private void
        Stream(ArrayList data, MaterialFilter matFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(MaterialFilter)));

            // Nothing at this level yet!           
        }

        private void
        Stream(ArrayList data, ParameterFilter matFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(ParameterFilter)));

            // Nothing at this level yet!           
        }

        private void
        Stream(ArrayList data, StructuralTypeFilter structFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(StructuralTypeFilter)));

            // Nothing at this level yet!           
        }

        private void
        Stream(ArrayList data, SymbolFilter symFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(SymbolFilter)));

            // Nothing at this level yet!           
        }

        private void
        Stream(ArrayList data, TypeFilter typeFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(TypeFilter)));

            // Nothing at this level yet!           
        }

        private void
        Stream(ArrayList data, WallUsageFilter wallFilter)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(WallUsageFilter)));

            // Nothing at this level yet!           
        }

        private void
        Stream(ArrayList data, PaperSize paperSize)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(PaperSize)));

            data.Add(new Snoop.Data.String("Name", paperSize.Name));
        }

        private void
        Stream(ArrayList data, PaperSource paperSource)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(PaperSource)));

            data.Add(new Snoop.Data.String("Name", paperSource.Name));
        }

        private void
        Stream(ArrayList data, PrintSetup prnSetup)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(PrintSetup)));

            data.Add(new Snoop.Data.Object("Current print setting", prnSetup.CurrentPrintSetting));
            data.Add(new Snoop.Data.Enumerable("Paper sizes", prnSetup.PaperSizes));
            data.Add(new Snoop.Data.Enumerable("Paper sources", prnSetup.PaperSources));           
        }

        private void
        Stream(ArrayList data, PrintParameters prnParams)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(PrintParameters)));

            data.Add(new Snoop.Data.String("Color depth", prnParams.ColorDepth.ToString()));
            data.Add(new Snoop.Data.String("Hidden line views", prnParams.HiddenLineViews.ToString()));
            data.Add(new Snoop.Data.Bool("Hide crop boundaries", prnParams.HideCropBoundaries));
            data.Add(new Snoop.Data.Bool("Hide refor work planes", prnParams.HideReforWorkPlanes)); // TBD - Check property name (seems to be spelt wrong)
            data.Add(new Snoop.Data.Bool("Hide scope boxes", prnParams.HideScopeBoxes));
            data.Add(new Snoop.Data.Bool("Hide unreferenced view tags", prnParams.HideUnreferencedViewTages)); // TBD - Check property name (seems to be spelt wrong)
            data.Add(new Snoop.Data.String("Margin type", prnParams.MarginType.ToString()));
            data.Add(new Snoop.Data.String("Page orientation", prnParams.PageOrientation.ToString()));

            try {
                data.Add(new Snoop.Data.String("Paper placement", prnParams.PaperPlacement.ToString()));
            }
            catch (SystemException ex) {
                data.Add(new Snoop.Data.Exception("Paper placement", ex));
            }

            data.Add(new Snoop.Data.Object("Paper size", prnParams.PaperSize));
            data.Add(new Snoop.Data.Object("Paper source", prnParams.PaperSource));

            try {
                data.Add(new Snoop.Data.String("Raster quality", prnParams.RasterQuality.ToString()));
            }
            catch (SystemException ex) {
                data.Add(new Snoop.Data.Exception("Raster quality", ex));
            }

            try {
                data.Add(new Snoop.Data.Double("User defined margin X", prnParams.UserDefinedMarginX));
            }
            catch (SystemException ex) {
                data.Add(new Snoop.Data.Exception("User defined margin X", ex));
            }

            try {
                data.Add(new Snoop.Data.Double("User defined margin Y", prnParams.UserDefinedMarginY));
            }
            catch (SystemException ex) {
                data.Add(new Snoop.Data.Exception("User defined margin Y", ex));
            }

            data.Add(new Snoop.Data.Bool("View links in blue", prnParams.ViewLinksinBlue));

            try {
                data.Add(new Snoop.Data.Int("Zoom", prnParams.Zoom));
            }
            catch (SystemException ex) {
                data.Add(new Snoop.Data.Exception("Zoom", ex));
            }

            try {
                data.Add(new Snoop.Data.String("Zoom type", prnParams.ZoomType.ToString()));
            }
            catch (SystemException ex) {
                data.Add(new Snoop.Data.Exception("Zoom type", ex));
            }
        }

        private void
        Stream(ArrayList data, PostedError postErr)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(PostedError)));

            data.Add(new Snoop.Data.String("Descriptor", postErr.Descriptor));
            data.Add(new Snoop.Data.String("Error type", postErr.ErrorType.ToString()));
            data.Add(new Snoop.Data.String("Message", postErr.Message));            
            data.Add(new Snoop.Data.Int("Raw Id", postErr.RawID));
        }

        private void
        Stream(ArrayList data, VolumeCalculationSetting volCalc)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(VolumeCalculationSetting)));

            data.Add(new Snoop.Data.Object("Volume calculation options", volCalc.VolumeCalculationOptions));           
        }

        private void
        Stream(ArrayList data, VolumeCalculationOptions volCalcOpts)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(VolumeCalculationOptions)));

            data.Add(new Snoop.Data.String("Room area boundary location", volCalcOpts.RoomAreaBoundaryLocation.ToString()));
            data.Add(new Snoop.Data.Bool("Volume computation enable", volCalcOpts.VolumeComputationEnbale));              
        }

        private void
        Stream(ArrayList data, ViewSheetSetting viewSheetSetting)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(ViewSheetSetting)));

            data.Add(new Snoop.Data.Object("Available views", viewSheetSetting.AvailableViews));
            data.Add(new Snoop.Data.Object("Current view sheet set", viewSheetSetting.CurrentViewSheetSet));
        }

        private void
        Stream(ArrayList data, DialogBoxData dlgBoxData)
        {
            data.Add(new Snoop.Data.ClassSeparator(typeof(ViewSheetSetting)));

            data.Add(new Snoop.Data.Int("Help id", dlgBoxData.HelpId));           
        }
    }
}
